==> ace_5.4.2.1.0.orig.tar.gz: ACE_wrappers/TAO/TAO_IDL/util/utl_list.cpp <==
// utl_list.cpp,v 1.9 2003/11/04 05:21:30 dhinton Exp

/*

COPYRIGHT

Copyright 1992, 1993, 1994 Sun Microsystems, Inc.  Printed in the United
States of America.  All Rights Reserved.

This product is protected by copyright and distributed under the following
license restricting its use.

The Interface Definition Language Compiler Front End (CFE) is made
available for your use provided that you include this license and copyright
notice on all media and documentation and the software program in which
this product is incorporated in whole or part. You may copy and extend
functionality (but may not remove functionality) of the Interface
Definition Language CFE without charge, but you are not authorized to
license or distribute it to anyone else except as part of a product or
program developed by you or with the express written consent of Sun
Microsystems, Inc. ("Sun").

The names of Sun Microsystems, Inc. and any of its subsidiaries or
affiliates may not be used in advertising or publicity pertaining to
distribution of Interface Definition Language CFE as permitted herein.

This license is effective until terminated by Sun for failure to comply
with this license.  Upon termination, you shall destroy or return all code
and documentation for the Interface Definition Language CFE.

INTERFACE DEFINITION LANGUAGE CFE IS PROVIDED AS IS WITH NO WARRANTIES OF
ANY KIND INCLUDING THE WARRANTIES OF DESIGN, MERCHANTIBILITY AND FITNESS
FOR A PARTICULAR PURPOSE, NONINFRINGEMENT, OR ARISING FROM A COURSE OF
DEALING, USAGE OR TRADE PRACTICE.

INTERFACE DEFINITION LANGUAGE CFE IS PROVIDED WITH NO SUPPORT AND WITHOUT
ANY OBLIGATION ON THE PART OF Sun OR ANY OF ITS SUBSIDIARIES OR AFFILIATES
TO ASSIST IN ITS USE, CORRECTION, MODIFICATION OR ENHANCEMENT.

SUN OR ANY OF ITS SUBSIDIARIES OR AFFILIATES SHALL HAVE NO LIABILITY WITH
RESPECT TO THE INFRINGEMENT OF COPYRIGHTS, TRADE SECRETS OR ANY PATENTS BY
INTERFACE DEFINITION LANGUAGE CFE OR ANY PART THEREOF.

IN NO EVENT WILL SUN OR ANY OF ITS SUBSIDIARIES OR AFFILIATES BE LIABLE FOR
ANY LOST REVENUE OR PROFITS OR OTHER SPECIAL, INDIRECT AND CONSEQUENTIAL
DAMAGES, EVEN IF SUN HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.

Use, duplication, or disclosure by the government is subject to
restrictions as set forth in subparagraph (c)(1)(ii) of the Rights in
Technical Data and Computer Software clause at DFARS 252.227-7013 and FAR
52.227-19.

Sun, Sun Microsystems and the Sun logo are trademarks or registered
trademarks of Sun Microsystems, Inc.

SunSoft, Inc.
2550 Garcia Avenue
Mountain View, California  94043

NOTE:

SunOS, SunSoft, Sun, Solaris, Sun Microsystems or the Sun logo are
trademarks or registered trademarks of Sun Microsystems, Inc.

*/

// Implementation of generic single-linked list.

// NOTE: This list class only works correctly because we use single public
//       inheritance, as opposed to multiple inheritance or public virtual.
//	     It relies on a type-unsafe cast from UTL_List to subclasses, which
//	     will cease to operate correctly if you use either multiple or
//	     public virtual inheritance.

#include "utl_list.h"
#include "ace/OS_Memory.h"

ACE_RCSID (util, 
           utl_list, 
           "utl_list.cpp,v 1.9 2003/11/04 05:21:30 dhinton Exp")

UTL_List::UTL_List (UTL_List *c)
	: pd_cdr_data (c)
{
}

UTL_List::~UTL_List (void)
{
}

// Compute list length.
long
UTL_List::list_length (long n)
{
  if (this->pd_cdr_data == 0)
    {
      return n;
    }
  else
    {
      return this->pd_cdr_data->list_length (n + 1);
    }
}

// Smash last cdr with l.
void
UTL_List::nconc (UTL_List *l)
{
  if (this->pd_cdr_data == 0)
    {
      this->pd_cdr_data = l;
    }
  else
    {
      this->pd_cdr_data->nconc (l);
    }
}

// Override this operation to copy lists of other types.
UTL_List *
UTL_List::copy (void)
{
  UTL_List *retval = 0;

  if (this->pd_cdr_data == 0)
    {
      ACE_NEW_RETURN (retval,
                      UTL_List (0),
                      0);
    }
  else
    {
      ACE_NEW_RETURN (retval,
                      UTL_List (this->pd_cdr_data->copy ()),
                      0);
    }

  return retval;
}

// Get next list.
UTL_List *
UTL_List::tail (void)
{
  return pd_cdr_data;
}

// Set next list.
void
UTL_List::set_tail (UTL_List *l)
{
  this->pd_cdr_data->destroy ();
  this->pd_cdr_data = l;
}

// Compute list length.
long
UTL_List::length (void)
{
  return list_length (1);
}

void
UTL_List::destroy (void)
{
  if (this->pd_cdr_data != 0)
    {
      this->pd_cdr_data->destroy ();
      delete this->pd_cdr_data;
      this->pd_cdr_data = 0;
    }
}

// UTL_List active iterator.

UTL_ListActiveIterator::UTL_ListActiveIterator (UTL_List *s)
  : source (s)
{
}

// Is iterator done?
idl_bool
UTL_ListActiveIterator::is_done (void)
{
  return (this->source == 0) ? I_TRUE : I_FALSE;
}

// Advance to next item.
void
UTL_ListActiveIterator::next (void)
{
  if (this->source != 0)
    {
      this->source = this->source->tail ();
    }
}

==> ace_6.5.4.orig.tar.gz: ACE_wrappers/TAO/TAO_IDL/util/utl_list.cpp <==
/*

COPYRIGHT

Copyright 1992, 1993, 1994 Sun Microsystems, Inc.  Printed in the United
States of America.  All Rights Reserved.

This product is protected by copyright and distributed under the following
license restricting its use.

The Interface Definition Language Compiler Front End (CFE) is made
available for your use provided that you include this license and copyright
notice on all media and documentation and the software program in which
this product is incorporated in whole or part. You may copy and extend
functionality (but may not remove functionality) of the Interface
Definition Language CFE without charge, but you are not authorized to
license or distribute it to anyone else except as part of a product or
program developed by you or with the express written consent of Sun
Microsystems, Inc. ("Sun").

The names of Sun Microsystems, Inc. and any of its subsidiaries or
affiliates may not be used in advertising or publicity pertaining to
distribution of Interface Definition Language CFE as permitted herein.

This license is effective until terminated by Sun for failure to comply
with this license.  Upon termination, you shall destroy or return all code
and documentation for the Interface Definition Language CFE.

INTERFACE DEFINITION LANGUAGE CFE IS PROVIDED AS IS WITH NO WARRANTIES OF
ANY KIND INCLUDING THE WARRANTIES OF DESIGN, MERCHANTIBILITY AND FITNESS
FOR A PARTICULAR PURPOSE, NONINFRINGEMENT, OR ARISING FROM A COURSE OF
DEALING, USAGE OR TRADE PRACTICE.

INTERFACE DEFINITION LANGUAGE CFE IS PROVIDED WITH NO SUPPORT AND WITHOUT
ANY OBLIGATION ON THE PART OF Sun OR ANY OF ITS SUBSIDIARIES OR AFFILIATES
TO ASSIST IN ITS USE, CORRECTION, MODIFICATION OR ENHANCEMENT.

SUN OR ANY OF ITS SUBSIDIARIES OR AFFILIATES SHALL HAVE NO LIABILITY WITH
RESPECT TO THE INFRINGEMENT OF COPYRIGHTS, TRADE SECRETS OR ANY PATENTS BY
INTERFACE DEFINITION LANGUAGE CFE OR ANY PART THEREOF.

IN NO EVENT WILL SUN OR ANY OF ITS SUBSIDIARIES OR AFFILIATES BE LIABLE FOR
ANY LOST REVENUE OR PROFITS OR OTHER SPECIAL, INDIRECT AND CONSEQUENTIAL
DAMAGES, EVEN IF SUN HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.

Use, duplication, or disclosure by the government is subject to
restrictions as set forth in subparagraph (c)(1)(ii) of the Rights in
Technical Data and Computer Software clause at DFARS 252.227-7013 and FAR
52.227-19.

Sun, Sun Microsystems and the Sun logo are trademarks or registered
trademarks of Sun Microsystems, Inc.

SunSoft, Inc.
2550 Garcia Avenue
Mountain View, California  94043

NOTE:

SunOS, SunSoft, Sun, Solaris, Sun Microsystems or the Sun logo are
trademarks or registered trademarks of Sun Microsystems, Inc.

*/

// Implementation of generic single-linked list.

// NOTE: This list class only works correctly because we use single public
//       inheritance, as opposed to multiple inheritance or public virtual.
//       It relies on a type-unsafe cast from UTL_List to subclasses, which
//       will cease to operate correctly if you use either multiple or
//       public virtual inheritance.

#include "utl_list.h"
#include "ace/OS_Memory.h"

UTL_List::UTL_List (UTL_List *c)
  : pd_cdr_data (c)
{
}

UTL_List::~UTL_List (void)
{
}

// Compute list length.
ACE_CDR::Long
UTL_List::list_length (ACE_CDR::Long n)
{
  if (this->pd_cdr_data == 0)
    {
      return n;
    }
  else
    {
      return this->pd_cdr_data->list_length (n + 1);
    }
}

// Smash last cdr with l.
void
UTL_List::nconc (UTL_List *l)
{
  if (this->pd_cdr_data == 0)
    {
      this->pd_cdr_data = l;
    }
  else
    {
      this->pd_cdr_data->nconc (l);
    }
}

// Override this operation to copy lists of other types.
UTL_List *
UTL_List::copy (void)
{
  UTL_List *retval = 0;

  if (this->pd_cdr_data == 0)
    {
      ACE_NEW_RETURN (retval,
                      UTL_List (0),
                      0);
    }
  else
    {
      ACE_NEW_RETURN (retval,
                      UTL_List (this->pd_cdr_data->copy ()),
                      0);
    }

  return retval;
}

// Get next list.
UTL_List *
UTL_List::tail (void)
{
  return pd_cdr_data;
}

// Set next list.
void
UTL_List::set_tail (UTL_List *l)
{
  this->pd_cdr_data->destroy ();
  delete this->pd_cdr_data;
  this->pd_cdr_data = l;
}

// Compute list length.
ACE_CDR::Long
UTL_List::length (void)
{
  return list_length (1);
}

void
UTL_List::destroy (void)
{
  if (this->pd_cdr_data != 0)
    {
      this->pd_cdr_data->destroy ();
      delete this->pd_cdr_data;
      this->pd_cdr_data = 0;
    }
}

// UTL_List active iterator.

UTL_ListActiveIterator::UTL_ListActiveIterator (UTL_List *s)
  : source (s)
{
}

// Is iterator done?
bool
UTL_ListActiveIterator::is_done (void)
{
  return (this->source == 0) ? true : false;
}

// Advance to next item.
void
UTL_ListActiveIterator::next (void)
{
  if (this->source != 0)
    {
      this->source = this->source->tail ();
    }
}
